package edu.umd.cs.findbugs.formatStringChecker;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.IllegalFormatConversionException;
import java.util.MissingFormatArgumentException;
import java.util.UnknownFormatConversionException;

import org.junit.Test;

public class FormatterRuntimeTest {

	@Test
	public void shouldWork() {
		String.format("%d%n%d", 42, (short) 42);

		String.format("%d\n", new BigInteger("42"));
		String.format("%f\n", new BigDecimal(42));

	}

	@Test(expected = IllegalFormatConversionException.class)
	public void stringWhereIntegerExpected() {
		String.format("%d", "test");
	}

	@Test(expected = MissingFormatArgumentException.class)
	public void notEnoughParameters() {
		String.format("%s%s", "test");
	}

	public void passingAnArray() {
		System.out.printf("%s", new int[] { 42, 17 });
	}

	public void passingAnIntToABoolean() {
		System.out.printf("%b", 0);
	}

	@Test(expected = UnknownFormatConversionException.class)
	public void formatDateWithY() {
		String.format("%Y", new Date());
	}
	
	@Test
	public void testBug1874856FalsePositive() {
		// None of these should yield warnings
		Calendar c = new GregorianCalendar(1993, 4, 23);
		String s1 = String.format("s1 Duke's Birthday: %1$tm %1$te, %1$tY", c);
		System.out.println(s1);
		String s2 = String.format("s2 Duke's Birthday: %1$tm %<te, %<tY", c);
		System.out.println(s2);
		String s3 = String.format("s3 Duke's Birthday: %2$tm %<te, %<tY", c, c);
		System.out.println(s3);
		String s4 = String.format("s4 Duke's Birthday: %2$tm %<te, %te %<tY %te", c, c);
		System.out.println(s4);
		String s6 = String.format("s6 Duke's Birthday: %1.1f %2$te, %1$f", 1.0, c);
		System.out.println(s6);
	}
	
	@Test(expected=MissingFormatArgumentException.class)
	public void testBug1874856TruePositive() {
		Calendar c = new GregorianCalendar(1993, 4, 23);
			// Actually, this one should generate a warning
		String s5 = String.format("s5 Duke's Birthday: %<te, %te %<tY %te %12$tm ", 
				   c, c, c, c, c, c, c, c, c, c, c, c);
		System.out.println(s5);
		
	}
	
	@Test(expected = IllegalFormatConversionException.class)
	public void testDateMismatch() {
		System.out.printf("%tY\n", "2008");
	}

}
